#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "hexchat-plugin.h"

static hexchat_plugin *ph;
static int drakcowsay_timeout = 1000;
static char *fifo_name = ".drakcowsay-fifo";
static char *fifo_path;
static int fifo_fd;

static int drakcowsay_timeout_cb(void *userdata) {
    char buf[8192];
    int r;

    while ((r = read(fifo_fd, buf, sizeof(buf)-1)) > 0) {
        char *s = buf, *l;
        buf[buf[r-1] == '\n' ? r-1 : r] = '\0';
        while ((l = strsep(&s, "\n")))
          hexchat_commandf(ph, "SAY %s", l);
    }

    /* error or end of cowsay, reopen fifo and get the same fd (no need to rehook it) */
    close(fifo_fd);
    fifo_fd = open(fifo_path, O_RDONLY | O_NONBLOCK);

    return 1; // return 1 to keep the timer
}

int hexchat_plugin_init(hexchat_plugin *plugin_handle,
                      char **plugin_name,
                      char **plugin_desc,
                      char **plugin_version,
                      char *arg) {
    char *home;

    ph = plugin_handle;

    *plugin_name = "drakcowsay";
    *plugin_desc = "Prints ASCII-art rendered by drakcowsay";
    *plugin_version = "0.1";


    home = getenv("HOME");
    fifo_path = malloc(strlen(home) + strlen(fifo_name) + 2);
    strcpy(fifo_path, home);
    strcat(fifo_path, "/");
    strcat(fifo_path, fifo_name);
    unlink(fifo_path);
    mkfifo(fifo_path, 0666);
    fifo_fd = open(fifo_path, O_RDONLY | O_NONBLOCK);

    hexchat_hook_timer(ph, drakcowsay_timeout, drakcowsay_timeout_cb, NULL);
    //    hexchat_hook_fd(ph, fifo_fd, HEXCHAT_FD_READ | HEXCHAT_FD_WRITE | HEXCHAT_FD_EXCEPTION, drakcowsay_fifo_cb, NULL); 

    hexchat_print(ph, "drakcowsay plugin loaded successfully\n");

    return 1;
}

int hexchat_plugin_deinit(hexchat_plugin *plugin_handle) {
    close(fifo_fd);
    free(fifo_path);
    hexchat_print(ph, "drakcowsay plugin unloaded, bye\n");
    return 1;
}
